package adr.custom.converters;

import java.text.ParseException;

import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.ConverterException;
import javax.swing.text.MaskFormatter;

public class MaskConverter implements javax.faces.convert.Converter, StateHolder{

	/**
     * <p>The standard converter id for this converter.</p>
     */
    public static final String CONVERTER_ID = "adr.custom.Mask";
    
    /** 
     * The valid special characters to form mask string. It is used by javax.swing.text.MaskFormatter for masking. 
     * Example: ##-###-##### will mask the number as 12-123-1234. 
     * Character  	    Description
     * ---------------------------------------------------------------------------------------------------
     * # 		        Any valid number (Character.isDigit). 
     * ' (single quote) Escape character, used to escape any of the special formatting characters. 
     * U 		        Any character (Character.isLetter). All lowercase letters are mapped to uppercase. 
     * L                Any character (Character.isLetter). All uppercase letters are mapped to lowercase. 
     * A                Any character or number (Character.isLetter or Character.isDigit). 
     * ?                Any character (Character.isLetter). 
     * *                Anything. 
     * H 		        Any hex character (0-9, a-f or A-F). 
     */
    private String mask = null;
    private String placeholderString = null;
    private Character placeholder;
    private Boolean literal = false; 
    
	@Override
	public Object getAsObject(FacesContext context, UIComponent component,
			String value) {
		
        if (context == null || component == null) {
            throw new NullPointerException();
        }
        
		if(value == null){
			return null;
		}
		
		value = value.trim();
		if(value.isEmpty()){
			return null;
		}

		Object result = null;
		try {
			result = stringToValue(value);
		} catch (ParseException e) {
//			e.printStackTrace();
			throw new ConverterException(e);
		}
		
		return result;
	}

	private Object stringToValue(String value) throws ParseException {
		if(mask == null){
			//TODO:
		}
		MaskFormatter formatter = new MaskFormatter(mask.trim()); //Extra Precaution.
			formatter.setValueClass(String.class);
			formatter.setValueContainsLiteralCharacters(literal);
		return formatter.stringToValue(value);
	}

	@Override
	public String getAsString(FacesContext context, UIComponent component,
			Object value) {
        if (context == null || component == null) {
            throw new NullPointerException();
        }
        
		if(value == null){
			return null;
		}
		
/*		if(!(value instanceof String)){
			return null;
		}*/

		if(!isValidMask()){
			//TODO: Decide to throw null or empty String.
		}
		
		String result = null;
		try {
			result = valueToString(value);
		} catch (ParseException e) {
//			e.printStackTrace();
			throw new ConverterException(e);
		}
		
		return result;
	}

	private String valueToString(Object value) throws ParseException {
		if(mask == null){
			//TODO:
		}
		MaskFormatter formatter = new MaskFormatter(mask.trim()); //Extra Precaution.
			formatter.setValueClass(String.class);
			formatter.setValueContainsLiteralCharacters(false);
			formatter.setPlaceholderCharacter(placeholder);
			formatter.setPlaceholder(placeholderString);
		return formatter.valueToString(value);
	}

	private boolean isValidMask(){
		
		if(mask == null){
			return false;
		}
		
		if(mask.trim().isEmpty()){
			return false;
		}
		
		//TODO: Decide if <placeholder>/<placeholderString> is required. Mandatory? 
		
		return true;
	}

	public String getMask() {
		return mask;
	}

	public void setMask(String mask) {
		this.mask = mask.trim();
	}

	public boolean isLiteral() {
		return literal;
	}

	public void setLiteral(boolean literal) {
		this.literal = literal;
	}

	public String getPlaceholderString() {
		return placeholderString;
	}

	public void setPlaceholderString(String placeholderString) {
		this.placeholderString = placeholderString;
	}

	public char getPlaceholder() {
		return placeholder;
	}

	public void setPlaceholder(char placeholder) {
		this.placeholder = placeholder;
	}

	@Override
	public void restoreState(FacesContext context, Object state) {
		Object values[] = (Object[]) state;
			mask = (String)values[0]; 
			placeholderString = (String)values[1]; 
			placeholder = (Character)values[2]; 
			literal = (Boolean)values[3]; 
	}

	@Override
	public Object saveState(FacesContext context) {
		Object values[] = new Object[4];
			values[0] = mask;
			values[1] = placeholderString;
			values[2] = placeholder;
			values[3] = literal;
		return values;
	}


	private boolean transientFlag = false;

	@Override
	public boolean isTransient() {
		return transientFlag;
	}

	@Override
	public void setTransient(boolean transientFlag) {
		this.transientFlag = transientFlag;
	}


}
