package adr.custom.tags;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.servlet.jsp.JspException;

import adr.custom.converters.MaskConverter;

import com.sun.faces.el.ELUtils;
import com.sun.faces.taglib.jsf_core.AbstractConverterTag;

public class MaskConverterTag extends AbstractConverterTag {
	private static final long serialVersionUID = -8236402321933243571L;
	private static ValueExpression CONVERTER_ID_EXPR = null;

	private String mask;
	private String placeholderString;
	private char placeholder;
	private boolean literal;
	
	private ValueExpression maskExpression;
	private ValueExpression placeholderStringExpression;
	private ValueExpression placeholderExpression;
	private ValueExpression literalExpression;

    private boolean placeholderStringSpecified;
    private boolean placeholderSpecified;

	public MaskConverterTag() {
		super();
		init();
	}

	private void init() {
		mask = null;
		placeholderString = null;
		literal = false;
		
		maskExpression = null;
		placeholderStringExpression = null;
		placeholderExpression = null;
		literalExpression = null;

		placeholderStringSpecified = false;
		placeholderSpecified = false;
		
		
		if (CONVERTER_ID_EXPR == null) { 
			FacesContext context = FacesContext.getCurrentInstance(); 
			ExpressionFactory factory = context.getApplication() .getExpressionFactory(); 
			CONVERTER_ID_EXPR = factory.createValueExpression(context .getELContext(),
								"adr.custom.Mask", String.class); 
		 }
		 
	}

	@Override
	public int doStartTag() throws JspException {
		super.setConverterId(CONVERTER_ID_EXPR);
		return super.doStartTag();
	}

	@Override
	protected Converter createConverter() throws JspException {
		MaskConverter result = (MaskConverter) super.createConverter();
		
		evaluateExpressions();
		
		result.setMask(mask);
		result.setLiteral(literal);
		if(placeholderSpecified){
			result.setPlaceholder(placeholder);
		}
		if(placeholderStringSpecified){
			result.setPlaceholderString(placeholderString);
		}
		
		return result;
	}

	private void evaluateExpressions() {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        ELContext elContext = facesContext.getELContext();

        if (maskExpression != null) {
            mask = (String)
            ELUtils.evaluateValueExpression(maskExpression,
                elContext);
        }

        if (placeholderExpression != null) {
        	placeholder = (Character)
            ELUtils.evaluateValueExpression(placeholderExpression,
                elContext);
        }
        
        if (placeholderStringExpression != null) {
        	placeholderString = (String)
            ELUtils.evaluateValueExpression(placeholderStringExpression,
                elContext);
        }
        
        if (literalExpression != null) {
            if (literalExpression.isLiteralText()) {
                literal =
                Boolean.valueOf(
                		literalExpression.getExpressionString()).
                    booleanValue();
            } else {
            	literal = ((Boolean)
                                   ELUtils.evaluateValueExpression(literalExpression,
                                       elContext)).booleanValue();
            }
        }

		
	}

	public ValueExpression getMaskExpression() {
		return maskExpression;
	}

	public void setMaskExpression(ValueExpression maskExpression) {
		this.maskExpression = maskExpression;
	}

	public ValueExpression getPlaceholderStringExpression() {
		return placeholderStringExpression;
	}

	public void setPlaceholderStringExpression(
			ValueExpression placeholderStringExpression) {
		this.placeholderStringExpression = placeholderStringExpression;
		this.placeholderStringSpecified = true;
	}

	public ValueExpression getPlaceholderExpression() {
		return placeholderExpression;
	}

	public void setPlaceholderExpression(ValueExpression placeholderExpression) {
		this.placeholderExpression = placeholderExpression;
		this.placeholderSpecified = true;
	}

	public ValueExpression getLiteralExpression() {
		return literalExpression;
	}

	public void setLiteralExpression(ValueExpression literalExpression) {
		this.literalExpression = literalExpression;
	}
}
